Object subclass:#DataSetLabel
	instanceVariableNames:'label selector argument adjust font fgColor bgColor layout
		preferredExtent'
	classVariableNames:''
	poolDictionaries:''
	category:'Views-DataSet'
!


!DataSetLabel class methodsFor:'instance creation'!

columnDescription:aDescription builder:aBuilder on:aGC
    ^ self new columnDescription:aDescription builder:aBuilder on:aGC

! !

!DataSetLabel methodsFor:'accessing colors'!

backgroundColor
    "returns the background color of the column or nil
    "
    ^ bgColor


!

foregroundColor
    "returns the foreground color of the column or nil
    "
    ^ fgColor

! !

!DataSetLabel methodsFor:'accessing dimensions'!

layout
    "returns the item's layout, a Rectangle
    "
    ^ layout


!

layout:aLayout
    "set the item's layout, a Rectangle
    "
    layout := aLayout


!

preferredExtent
    "returns the preferred extent of the label
    "
    ^ preferredExtent


! !

!DataSetLabel methodsFor:'actions'!

sendClickMsgTo:aReceiver
    "inform the receiver of a button release notification
    "
    (aReceiver notNil and:[selector notNil]) ifTrue:[
        selector numArgs ~~ 0 ifTrue:[
            aReceiver perform:selector with:argument
        ] ifFalse:[
            aReceiver perform:selector
        ]
    ]


! !

!DataSetLabel methodsFor:'drawing'!

drawLabel:aLabel atX:xLeft y:yTop hInset:hInset on:aGC
    "draw label
    "
    |x y|

    adjust == #left ifTrue:[
        x := xLeft + hInset.
    ] ifFalse:[
        x := layout width - (aLabel widthOn:aGC).

        adjust == #right ifTrue:[x := x - hInset]
                        ifFalse:[x := x // 2].
        x := xLeft + x.
    ].

    aLabel isImageOrForm ifFalse:[
        y := yTop + (font ascent)
    ] ifTrue:[
        y := yTop
    ].
    aLabel displayOn:aGC x:x y:y.
!

redrawLabelOn:aGC hInset:hInset
    "redraw the label; the background is cleared and the paint is set
    "
    |y|

    label isNil ifTrue:[ ^ self ].

    y := layout top + (layout height - preferredExtent y // 2).
    aGC font:font.

    (label isSequenceable and:[label isString not]) ifFalse:[
        self drawLabel:label atX:(layout left) y:y hInset:hInset on:aGC.
    ] ifTrue:[
        label do:[:el|
            el notNil ifTrue:[
                self drawLabel:el atX:(layout left) y:y hInset:hInset on:aGC.
                y := y + (el heightOn:aGC)
            ]
        ]
    ]


! !

!DataSetLabel methodsFor:'instance creation'!

columnDescription:aDescription builder:aBuilder on:aGC
    |device x y|

    label    := self resolveLabelFromDescription:aDescription withBuilder:aBuilder.
    selector := aDescription labelActionSelector.
    argument := (aDescription labelActionArgument) ? label.
    adjust   := (aDescription labelAlignment) ? #left.
    layout   := Rectangle left:0 right:0 top:0 bottom:0.
    fgColor  := aDescription labelForegroundColor.
    bgColor  := aDescription labelBackgroundColor.
    device   := aGC device.

    label isString ifTrue:[
        "/
        "/ check for empty label
        "/
        (label isEmpty or:[label withoutSeparators isEmpty]) ifTrue:[
            label := nil
        ]
    ].

    fgColor notNil ifTrue:[ fgColor := fgColor on:device ].
    bgColor notNil ifTrue:[ bgColor := bgColor on:device ].

    label isNil ifTrue:[
        preferredExtent := 0 @ 0.
        ^ self
    ].

    font := aDescription labelFont.

    font notNil ifTrue:[ font := font on:device ]
               ifFalse:[ font := aGC font ].

    aGC font:font.

    (label isSequenceable and:[label isString not]) ifFalse:[
        label := self label:label on:device.
        preferredExtent := Point x:(label widthOn:aGC) y:(label heightOn:aGC).
        ^ self
    ].
    x := y := 0.

    label := label collect:[:el||lbl|
        (lbl := self label:el on:device) notNil ifTrue:[
            x := (lbl widthOn:aGC) max:x.
            y := y + (lbl heightOn:aGC)
        ].
        lbl
    ].
    preferredExtent := Point x:x y:y.


! !

!DataSetLabel methodsFor:'notifications'!

originChanged:aDeltaPoint
    "called when the origin of the layout changed; scrolling left or right
    "
    |origin extent|

    origin := layout origin.
    extent := layout extent.

    layout origin:(origin - aDeltaPoint) extent:extent

! !

!DataSetLabel methodsFor:'private'!

label:aLabel on:aDevice
    "returns registered label on a device
    "
    |lbl img|

    aLabel isString ifFalse:[
        aLabel isImageOrForm ifTrue:[
            aLabel device ~~ aDevice ifTrue:[lbl := aLabel copy on:aDevice]
                                    ifFalse:[lbl := aLabel].
            lbl isImage ifTrue:[
                lbl clearMaskedPixels
            ].
            ^ lbl
        ].
        aLabel class == LabelAndIcon ifTrue:[
            lbl := aLabel on:aDevice.
            (img := lbl image) isImage ifTrue:[img clearMaskedPixels].
            (img := lbl icon)  isImage ifTrue:[img clearMaskedPixels].
            ^ lbl
        ]
    ].
    ^ aLabel
!

resolveLabelFromDescription:aDescription withBuilder:aBuilder

    |label appl size lbl|

    aBuilder isNil ifTrue:[
        ^ aDescription rawLabel        
    ].
    label := aDescription label.

    label isString ifTrue:[
        aDescription labelIsImage ifTrue:[
            (    aBuilder isEditing
             or:[(label := aBuilder labelFor:(label asSymbol)) isNil]
            ) ifTrue:[
                ^ aDescription class defaultIcon
            ].
            ^ label
        ].

        aDescription translateLabel == true ifTrue:[
            (    aBuilder isEditing
             or:[(appl := aBuilder application) isNil]
            ) ifTrue:[
                ^ label
            ].
            (    (lbl := appl resources string:label) isNil
             or:[(lbl := aBuilder aspectAt:(label asSymbol)) isNil]
            ) ifFalse:[
                label := lbl
            ]
        ]
    ].

    label isSequenceable ifTrue:[
        (size := label size) == 0 ifTrue:[              "/ empty label
            ^ nil
        ].
        (size == 1 and:[label isString not]) ifTrue:[   "/ collection with one entry
            ^ label at:1                                "/ not a string object
        ].
    ].
    ^ label



! !

!DataSetLabel methodsFor:'queries'!

isSelectable
    "returns true if the item is selectable; a valid selector
     to notify the receiver for a release button event exists
    "
    ^ selector notNil


! !

!DataSetLabel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/DataSetLabel.st,v 1.2 1999-03-19 21:22:35 cg Exp $'
! !
